<?php
/**
 * Tweets: a simple PHP Twitter library
 * @author Julien 'delphiki' Villetorte
 * http://www.lackofinspiration.com
 * http://twitter.com/delphiki
 */

class Tweet{
	var $text;
	var $parsed_text;
	var $date;
	var $formated_date;
	var $status_id;
	var $source;
	var $reply_to;
	
	function __construct($status){
		$this->text = $status->getElementsByTagName('text')->item(0)->nodeValue;
		$this->date = $status->getElementsByTagName('created_at')->item(0)->nodeValue;
		$this->status_id = $status->getElementsByTagName('id')->item(0)->nodeValue;
		$this->source = $status->getElementsByTagName('source')->item(0)->nodeValue;
		$this->reply_to = $status->getElementsByTagName('in_reply_to_screen_name')->item(0)->nodeValue;
		
		$this->parse();
	}
	
	/**
	 * Parse les messages en html 
	 */
	function parse(){
		$this->parsed_text = $this->text;
		$this->parsed_text = preg_replace('#http://[a-z0-9._/-]+#i', '<a href="$0">$0</a>', $this->parsed_text);
		$this->parsed_text = preg_replace('#@([a-z0-9_]+)#i', '@<a href="http://twitter.com/$1">$1</a>', $this->parsed_text);
		$this->parsed_text = preg_replace('# \#([a-z0-9_-]+)#i', ' #<a href="http://search.twitter.com/search?q=%23$1">$1</a>', $this->parsed_text);
	}
	
	/**
	 * Formatte la date selon le format passé en paramètre
	 * @param string $format
	 */
	function format($format){
		$datetime = date_create($this->date);
		$timestamp = $datetime->format('U');
		$this->formated_date = date($format, $timestamp);
	}
}

class TwitterUser{
	var $id;
	var $screen_name;
	var $name;
	var $location;
	var $description;
	var $profile_image;
	var $website;
	var $followers_count;
	var $following_count;
	var $registration_date;
	var $f_registration_date;
	var $statuses_count;
	
	var $date_format = 'd M Y, H:i:s';
		
	function __construct($username){
		$this->screen_name = $username;
	}
	
	/**
	 * Charge les données sur l'utilisateur
	 */
	function load(&$dom){
		$this->id = $dom->getElementsByTagName('id')->item(0)->nodeValue;
		$this->screen_name = $dom->getElementsByTagName('screen_name')->item(0)->nodeValue;
		$this->name = $dom->getElementsByTagName('name')->item(0)->nodeValue;
		$this->location = $dom->getElementsByTagName('location')->item(0)->nodeValue;
		$this->description = $dom->getElementsByTagName('description')->item(0)->nodeValue;
		$this->profile_image = $dom->getElementsByTagName('profile_image_url')->item(0)->nodeValue;
		$this->website = $dom->getElementsByTagName('url')->item(0)->nodeValue;
		$this->followers_count = $dom->getElementsByTagName('followers_count')->item(0)->nodeValue;
		$this->following_count = $dom->getElementsByTagName('friends_count')->item(0)->nodeValue;
		$this->registration_date = $dom->getElementsByTagName('created_at')->item(0)->nodeValue;
		$this->statuses_count = $dom->getElementsByTagName('statuses_count')->item(0)->nodeValue;
		
		$this->format();
	}
	
	/**
	 * Formatte la date selon le format passé en paramètre
	 * @param string $format[optional]
	 */	
	function format($format = NULL){
		if($format === NULL)
			$format = $this->date_format;
		else
			$this->date_format = $format;
		$datetime = date_create($this->registration_date);
		$timestamp = $datetime->format('U');
		$this->f_registration_date = date($format, $timestamp);
	}
}

class Tweets{
	var $username;
	var $user;
	var $statuses = array();
	var $length = 0;
	var $date_format = 'd M Y, H:i:s';
	var $utf8_display;
	var $c_path = './tw_cache/';
	var $c_lifetime = 5;
	var $errors = array();
	
	/**
	 * @param string $username Nom d'utilisateur Twitter
	 * @param bool $utf8_display[optional] Effectue la conversion UTF8 -> ISO-8859-1 si false
	 */
	function __construct($username, $utf8_display = true){
		$this->user = new TwitterUser($username);
		$this->username = &$this->user->screen_name;
		$this->utf8_display = $utf8_display;
		
		if(!file_exists($this->c_path))
			mkdir($this->c_path, 0755);
		
		$this->load();
	}
	
	/**
	 * Charge les données (du cache ou de Twitter) dans un DOMDocument
	 */
	function load(){
		$dom = new DOMDocument('1.0', 'UTF-8');

		if(file_exists($this->c_path.$this->username.'.xml') && $this->c_lifetime > 0){
			$stat = stat($this->c_path.$this->username.'.xml');
			if(time() > ($stat['mtime'] + ($this->c_lifetime * 60)))
				$data = $this->get_external_xml();
			else
				$dom->load($this->c_path.$this->username.'.xml');
		}
		else
			$data = $this->get_external_xml();
		
		if(isset($data) && ($data === false || empty($data)))
			$this->errors[] = 'Erreur lors du chargement.';
		elseif(isset($data)){
			$dom->loadXML($data);
			if($this->c_lifetime > 0)
				file_put_contents($this->c_path.$this->username.'.xml', $data);
		}
		
		$statuses = $dom->getElementsByTagName('status');
		$user = $dom->getElementsByTagName('user')->item(0);
		$this->user->load($user);
		
		$this->length = $statuses->length;
		foreach($statuses as $status){
			$this->statuses[] = new Tweet($status);
		}
	}
	
	/**
	 * Charge les données depuis le fichier xml (situé sur Twitter) via cURL
	 * @return string
	 */
	function get_external_xml(){
		$ch = curl_init(); 
		$url = 'http://twitter.com/statuses/user_timeline.xml?screen_name='.$this->user->screen_name;
		curl_setopt($ch, CURLOPT_URL, $url); 
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); 
		curl_setopt($ch, CURLOPT_FAILONERROR, true); 
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
		curl_setopt($ch, CURLOPT_TIMEOUT, 3);
		$data = curl_exec($ch);
		curl_close($ch);
		
		return $data;
	}
	
	/**
	 * Permet de changer le nom d'utilisateur
	 * @param string $username Nom d'utilisateur Twitter
	 */
	function set_username($username){
		if(count($this->errors) > 0)
			return;
		$this->user = new TwitterUser($username);
		$this->load();
	}

	/**
	 * Permet de modifier le formattage de la date par défaut ('d M Y, H:i:s')
	 * @param string $format
	 */
	function set_date_format($format){
		$this->date_format = $format;
		$this->user->format($format);
	}

	/**
	 * Active ou désactive la conversion UTF8 -> ISO-8859-1
	 * @param bool $utf8
	 */
	function set_utf8_display($utf8){
		$this->utf8_display = $utf8;
	}
	
	/**
	 * Retourne le Tweet correspondant, après avoir formaté la date
	 * @return Tweet
	 * @param int $id Index dans $this->statuses
	 */
	function status($id){
		if(count($this->errors) > 0)
			return;
		$this->statuses[$id]->format($this->date_format);
		if(!$this->utf8_display){
			$this->statuses[$id]->text = utf8_decode($this->statuses[$id]->text);
			$this->statuses[$id]->parsed_text = utf8_decode($this->statuses[$id]->parsed_text);
		}
		return $this->statuses[$id];
	}
	
	/**
	 * Permet de modifier le dossier qui contient le cache
	 * @param string $path Chemin vers le dossier de cache
	 */
	function set_cache_path($path){
		$this->c_path = $path;
		if(!file_exists($this->c_path))
			mkdir($this->c_path, 0755);
	}
	
	/**
	 * Permet de modifier la fréquence de rafraichissement du cache, une valeur inférieure ou égale à 0 désactive la mise en cache
	 * @param int $time Nombre de minutes après lequel le cache expire
	 */
	function set_cache_lifetime($time){
		$this->c_lifetime = (int)$time;
	}
}